import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Scanner;

import javax.imageio.ImageIO;

public class Mandelbrot {
	public static final int HEIGHT = 800;

	public static final int WIDTH = 800;

	public static final int THRESHOLD = 2000;

	public static void main(String[] args) throws InterruptedException, IOException {
		double x1 = -2.0;
		double y1 = -2.0;
		double x2 = 2.0;
		double y2 = 2.0;
		int colorChoice = 1;
		int numThreads = 1;
		String fileName = "output.png";
		
		// Uncomment to allow user to enter coordinates
		//Scanner keyboard = new Scanner(System.in);
		//
		//System.out.println("Please enter coordinates of region to render:");
		//System.out.print("  x1: ");
		//x1 = keyboard.nextDouble();
		//System.out.print("  y1: ");
		//y1 = keyboard.nextDouble();
		//System.out.print("  x2: ");
		//x2 = keyboard.nextDouble();
		//System.out.print("  y2: ");
		//y2 = keyboard.nextDouble();
		
		// Create bounding rectangle for Mandelbrot region
		Rect bounds = new Rect(x1,y1,x2,y2);

		// Uncomment to prompt user for output filename
		//System.out.print("Output filename (.png): ");
		//fileName = keyboard.next();
		
		// Uncomment to allow user to choose color scheme
		//System.out.println("Choose a color chooser: ");
		//System.out.println("   1 - Simple Gradient");
		//System.out.println("   2 - Color Mapping");
		//colorChoice = keyboard.nextInt();		

		// Uncomment to allow user to select number of threads
		//System.out.print("Enter the number of threads to use: ");
		//numThreads = keyboard.nextInt();
		
		System.out.println("Working...");
		
		// Create iteration count array and compute iterations
		int[][] iterCounts = new int[HEIGHT][WIDTH];
		computeIterations(bounds, numThreads, iterCounts);
		
		// Create output image from iteration array		
		BufferedImage bufferedImage = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_ARGB);
		renderImage(bufferedImage, iterCounts, colorChoice);

		// Write output image to PNG file
		OutputStream os = new BufferedOutputStream(new FileOutputStream(fileName));
		try {
			ImageIO.write(bufferedImage, "PNG", os);
		} finally {
			os.close();
		}
		
		System.out.println("Image written successfully!");
	}

	// Method to compute iterations for each pixel
	public static void computeIterations(Rect bounds, int numThreads, int[][] iterCounts) throws InterruptedException {
		// TODO: Create MandelbrotTasks and threads

		// TODO: Start threads

		// TODO: Wait for all threads to finish

	}

	// Method to create image buffer using user selected colorChooser
	public static BufferedImage renderImage(BufferedImage bufferedImage, int[][] iterCounts, int colorChoice) {
	
		ColorChooser chooser = null;
		
		// TODO: set chooser based on colorChoice menu selection
		if (colorChoice == 1) {
			chooser = new SimpleGradientColorChooser();
		} else if (colorChoice == 2) {
			chooser = new ColorMappingColorChooser(iterCounts);
		} else {
			chooser = new SimpleGradientColorChooser();
		}		
		
		// Use computed iteration counts to render the image
		Graphics g = bufferedImage.getGraphics();
		for (int i = 0; i < HEIGHT; i++) {
			for (int j = 0; j < WIDTH; j++) {
				Color c = chooser.getColor(iterCounts[i][j]);
				g.setColor(c);
				g.fillRect(j, i, 1, 1);
			}
		}
		g.dispose();
		return bufferedImage;
	}

}
